\documentclass[handout]{beamer}
%\documentclass[]{beamer}


%\usepackage[font=small,labelfont=bf,labelsep=none]{caption}
\usepackage{beamerthemeshadow}


\usetheme{Copenhagen}
\setbeamertemplate{navigation symbols}{}

%\setbeamertemplate{footline}{\hfill\insertframenumber/\inserttotalframenumber ~~~}
\setbeamertemplate{footline}{\hfill\insertframenumber~~~\vspace{1mm}}

\setbeamertemplate{itemize item}{$\Rightarrow$}


\usepackage[utf8]{inputenc}

\usepackage{tipa}

\usepackage{color}
\usepackage{xcolor}
%\usepackage{caption} 
\usepackage{graphics}

\usepackage{verbatim}

\usepackage{listings}
\usepackage{courier}

\usepackage[bookmarks=true]{hyperref}


\lstset{
         commentstyle=\color{blue},
         basicstyle=\scriptsize\ttfamily, % Standardschrift
         numbers=left,               % Ort der Zeilennummern
         numberstyle=\tiny,          % Stil der Zeilennummern
         %stepnumber=2,               % Abstand zwischen den Zeilennummern
         numbersep=8pt,              % Abstand der Nummern zum Text
         tabsize=2,                  % Groesse von Tabs
         extendedchars=true,         %
         breaklines=true,            % Zeilen werden Umgebrochen
         keywords={Cons},
         keywordstyle=\color{red},
    		frame=b,         
 %        keywordstyle=[1]\textbf,    % Stil der Keywords
 %        keywordstyle=[2]\textbf,    %
 %        keywordstyle=[3]\textbf,    %
 %        keywordstyle=[4]\textbf,   \sqrt{\sqrt{}} %
         stringstyle=\color{cyan}\ttfamily, % Farbe der String
         showspaces=false,           % Leerzeichen anzeigen ?
         showtabs=false,             % Tabs anzeigen ?
         showlines=true,
         xleftmargin=17pt,
         framexleftmargin=17pt,
         framexrightmargin=5pt,
         framexbottommargin=0pt,
         framextopmargin=20pt,
         %backgroundcolor=\color{lightgray},
         showstringspaces=false      % Leerzeichen in Strings anzeigen ?        
 }
\lstloadlanguages{% Check Dokumentation for further languages ...
         %[Visual]Basic
         %Pascal
         %C
         %C++
         %XML
         %HTML
         %Java
         Haskell
 }
    %\DeclareCaptionFont{blue}{\color{blue}} 

  %\captionsetup[lstlisting]{singlelinecheck=false, labelfont={blue}, textfont={blue}}

\usepackage{caption}
\DeclareCaptionFont{white}{\color{white}}
\DeclareCaptionFormat{listing}{\colorbox[cmyk]{0.43, 0.35, 0.35,0.01}{\parbox{\textwidth}{\hspace{15pt}#1#2#3}}}
\captionsetup[lstlisting]{format=listing,labelfont=white,textfont=white, singlelinecheck=false, margin=0pt, font={bf,footnotesize}}



% Folgendes sollte gelöscht werden, wenn man nicht am Anfang jedes
% Abschnitts die Gliederung nochmal sehen möchte.
%% \AtBeginSection[]
%% {
%%   \begin{frame}[t]{Gliederung}
%%     \tableofcontents[currentsection,currentsubsection,hideothersubsections]
%%   \end{frame}
%% }







\newenvironment{sample}[0]{\begin{exampleblock}{Beispiel:}}{\end{exampleblock}}






\begin{document}
\lstset{language=Haskell}

\title{GADTs}  
\subtitle{Generalisierte Algebraische Datentypen}
\author{Sven Urbanski (svenurbanski@gmail.com)}
\date{Sommersemester 2011}

\begin{comment} % nur für Compiler

> import Data.List
> import Control.Exception.Base

> intToFloat :: Integral a => a -> Float
> intToFloat a = fromInteger (toInteger a)

\end{comment}

\begin{frame}[fragile]
  \titlepage
\end{frame}



\begin{frame}[fragile]
    \tableofcontents
\end{frame}




\section{Einführung}

\subsection{Motivation}


\begin{frame}[fragile]{Motivation}

  \textbf{Generalisierte~Algebraische Datentypen}\\ (GADT, \textipa{[g2t]}) bieten:
  
  \begin{itemize}
  \item
    Formulierung von Invarianten im Typsystem
  \item
    Zusätzliche Typchecks
  \item
    weniger Laufzeitfehler  
  \end{itemize}
\end{frame}



\subsection{Von ADTs zu GADTs}

\begin{frame}[fragile]{ADT}

  \begin{itemize}
  \item
    \textbf{Algebraische Datentypen}\\ (\textipa{[a:de:te:]} auch \textbf{Summen-Datentypen}):
    \begin{sample}
      \begin{lstlisting}
< data X = X1 A | X2 B | ...
      \end{lstlisting}
    \end{sample}
  \item
    X ist der ADT\\
    X1, X2, ... sind Konstruktorfunktionen\\
    X $\approx$ die Menge aller Werte aus A, B, $\ldots$\\
    $X \approx \{a1, a2, a3, \ldots, b1, b2, b3, \ldots\}$\\
    $|X| = |A| + |B| + \ldots$
  \end{itemize}
\end{frame}



\begin{frame}[fragile]{Anwendungsfälle}

  \begin{itemize}
  \item
    beliebig viele
  \item
    überall, wo im Datentyp verschiedene Fälle unterschieden werden
    \begin{sample}
\begin{lstlisting}
< data Expr = IExpr Int
<           | BExpr Bool
<           | AndExpr Expr Expr
\end{lstlisting}

    \end{sample}
  \end{itemize}

\end{frame}



\section{Erweiterungen durch GADTs}
\subsection{Phantomtypen}

\begin{frame}[fragile]{Phantomtypen}

    \begin{exampleblock}{Normale Typparameter:}
      \begin{lstlisting}
< data List a = Nil
<             | Cons a (List a)
      \end{lstlisting}
    \end{exampleblock}

    \pause

    \begin{exampleblock}{Typparameter, die scheinbar nicht genutzt werden:}
      \begin{lstlisting}
> data PList a b = PNil
>                | PCons a (PList a b)
      \end{lstlisting}
    \end{exampleblock}

\end{frame}







\begin{frame}[fragile]{Phantomtypen}

    \begin{exampleblock}{Zur Erinnerung:}
      \begin{lstlisting}
< data PList a b = PNil
<                | PCons a (PList a b)
      \end{lstlisting}
    \end{exampleblock}

    Vorteil: Listen mit unterschiedlicher Semantik können nicht mehr gemischt werden
    \begin{sample}
      \begin{lstlisting}
< data Euro = ...
< data USD = ...
< euroToUsd :: PList a Euro -> PList a USD
      \end{lstlisting}
    \end{sample}


\end{frame}










\subsection{Typeinschränkungen}

\begin{frame}[fragile]{Einschränkungen der Typparameter}

    \begin{exampleblock}{In Java:}
\lstset{language=Java}
      \begin{lstlisting}
< interface NumList<A extends Number>
< class NCons<A> implements NumList<A>
< class NNil ...
      \end{lstlisting}
    \end{exampleblock}

\pause

    \begin{exampleblock}{In Haskell:}
      \begin{lstlisting}
> data NumList a = NNil
>                | Num a => NCons a (NumList a)
      \end{lstlisting}
    \end{exampleblock}

\end{frame}


\subsection{Erweiterung der Rückgabetypen}


\begin{frame}[fragile]{Syntax}
  
  \begin{exampleblock}{Alte Syntax:}
    \begin{lstlisting}
< data List a = Nil
<             | Cons a (List a)
    \end{lstlisting}
  \end{exampleblock}

\begin{exampleblock}{Neue Syntax:}
  \begin{lstlisting}
< data List a where
<   Nil :: List a
<   Cons :: a -> (List a) -> List a
  \end{lstlisting}
\end{exampleblock}


Nil ist eine Funktion ohne Parameter.\\
Cons ist eine Funktion mit zwei Parametern.\\
Gemeinsamkeiten?


\end{frame}




\begin{frame}[fragile]{Erweiterung durch GADTs}
  
  \begin{exampleblock}{Willkürliche Beispiele:}
    \begin{lstlisting}
< Cons1 :: a -> (List a) -> List a
< Cons2 :: a -> (List a) -> List (a,a)
< Cons3 :: a -> (List a) -> List (List a)
< Cons4 :: a -> (List a) -> a -- err
    \end{lstlisting}
  \end{exampleblock}

  \begin{itemize}
  \item
    Welchen Vorteil bringt das?
    \pause
  \item
    Mehr Typchecks
  \item
    Aber wie sinnvoll einsetzen?
  \end{itemize}

  
\end{frame}



\section{Beispiele für GADTs}
\subsection{Sichere Listen}

\begin{frame}[fragile]{Sichere Listen}

  \begin{itemize}
  \item
    Was bedeutet ``sicher''?
    \pause
  \item
    \textbf{Typ}-Unterscheidung zur Compilezeit:\\
    Leere / Nicht leere Listen
  \item
    Sinnvoll für partiell definierte Funktionen wie \texttt{head} und
    \texttt{tail}
  \end{itemize}
    \pause
  
  \begin{sample}
    \begin{lstlisting}
> data Safe a  = Safe a
> data NotSafe = NotSafe
> data SafeList a b where
>   Nil  :: SafeList a NotSafe
>   Cons :: a -> SafeList a b -> SafeList a (Safe b)
    \end{lstlisting}
  \end{sample}

\end{frame}



\begin{comment} % nur für Compiler

> sl2l :: SafeList a b -> [a]
> sl2l Nil = []
> sl2l (Cons e l) = e : sl2l l

> instance Show a => Show (SafeList a b) where
>     show Nil = "[]"
>     show l = "[" ++ intercalate ", " (map show l') ++ "]"
>         where
>           l' = sl2l l


\end{comment}


\begin{frame}[fragile]{Typ von sicheren Listen}

  \begin{exampleblock}{Zur Erinnerung:}
    \begin{lstlisting}
< data SafeList a b where
<   Nil  :: SafeList a NotSafe
<   Cons :: a -> SafeList a b -> SafeList a (Safe b)
    \end{lstlisting}
  \end{exampleblock}
  \begin{exampleblock}{Bedeutung der Typparameter:}
    \begin{lstlisting}
< SafeList a NotSafe                -- leere Liste
< SafeList a (Safe NotSafe)         -- Laenge == 1
< SafeList a (Safe b)               -- Laenge >= 1
< SafeList a (Safe (Safe NotSafe))  -- Laenge == 2
< SafeList a (Safe (Safe b))        -- Laenge >= 2
< SafeList a b                      -- Laenge beliebig
< ...                               -- Laenge n ???

    \end{lstlisting}
  \end{exampleblock}
\end{frame}


\begin{frame}[fragile]{Funktionen mit sicheren Listen: safeHead}

  \begin{exampleblock}{Zur Erinnerung:}
    \begin{lstlisting}
< data SafeList a b where
<   Nil  :: SafeList a NotSafe
<   Cons :: a -> SafeList a b -> SafeList a (Safe b)
    \end{lstlisting}
  \end{exampleblock}
  \begin{exampleblock}{Sicher:}
    \begin{lstlisting}
> safeHead :: SafeList a (Safe b) -> a
> safeHead (Cons x _) =  x
> -- safeHead Nil = ???
    \end{lstlisting}
  \end{exampleblock}
\end{frame}




\begin{frame}[fragile]{Funktionen mit sicheren Listen: unsafeHead}

  \begin{exampleblock}{Zur Erinnerung:}
    \begin{lstlisting}
< data SafeList a b where
<   Nil  :: SafeList a NotSafe
<   Cons :: a -> SafeList a b -> SafeList a (Safe b)
    \end{lstlisting}
  \end{exampleblock}
  \begin{exampleblock}{Unsicher:}
    \begin{lstlisting}
> unsafeHead :: SafeList a b -> a
> unsafeHead (Cons x _) =  x
> unsafeHead Nil = error "error in head!"
    \end{lstlisting}
  \end{exampleblock}

\end{frame}




\begin{frame}[fragile]{Funktionen mit sicheren Listen: tail}

  \begin{exampleblock}{Zur Erinnerung:}
    \begin{lstlisting}
< data SafeList a b where
<   Nil  :: SafeList a NotSafe
<   Cons :: a -> SafeList a b -> SafeList a (Safe b)
    \end{lstlisting}
  \end{exampleblock}
  \begin{sample}
    \begin{lstlisting}
> safeTail :: SafeList a (Safe b) -> SafeList a b
> safeTail (Cons _ t) = t
    \end{lstlisting}
  \end{sample}

 
\end{frame}



\begin{frame}[fragile]{Funktionen mit sicheren Listen: access}

  \begin{exampleblock}{Zur Erinnerung:}
    \begin{lstlisting}
< data SafeList a b where
<   Nil  :: SafeList a NotSafe
<   Cons :: a -> SafeList a b -> SafeList a (Safe b)
    \end{lstlisting}
  \end{exampleblock}

  \begin{exampleblock}{Sicherer Zugriff?}
    \begin{lstlisting}
< accessN :: Int
<            -> SafeList a (Safe (Safe (Safe ...)))
<            -> a
    \end{lstlisting}
  \end{exampleblock}
\end{frame}



\begin{frame}[fragile]{Funktionen mit sicheren Listen: map/fold}

  \begin{itemize}
  \item
    Was ist mit total definierten Funktionen wie map oder fold?
    \pause
  \item
    Kein Problem!
  \end{itemize}
    \begin{exampleblock}{map:}
      \begin{lstlisting}
> safeMap :: SafeList a c -> (a -> b) -> SafeList b c
> safeMap (Cons e t) f = Cons (f e) (safeMap t f)
> safeMap (Nil) _ = Nil
      \end{lstlisting}
    \end{exampleblock}
    \pause
    \begin{exampleblock}{fold:}
      \begin{lstlisting}
> fold :: (a -> b -> a) -> a -> SafeList b c -> a
> fold f start (Nil) = start
> fold f start (Cons e t) = fold f (f start e) t
      \end{lstlisting}
    \end{exampleblock}
\end{frame}








\begin{frame}[fragile]{Funktionen mit sicheren Listen: zip}

  \begin{exampleblock}{Zur Erinnerung:}
    \begin{lstlisting}
< data SafeList a b where
<   Nil  :: SafeList a NotSafe
<   Cons :: a -> SafeList a b -> SafeList a (Safe b)
    \end{lstlisting}
  \end{exampleblock}

  \begin{exampleblock}{zip:}
    \begin{lstlisting}
> safeZip :: SafeList a len -> SafeList b len
>         -> SafeList (a,b) len
> safeZip Nil Nil = Nil
> safeZip (Cons e1 l1) (Cons e2 l2) =
>     Cons (e1,e2) (safeZip l1 l2)
    \end{lstlisting}
  \end{exampleblock}

\end{frame}









\begin{frame}[fragile]{Funktionen mit sicheren Listen: Konvertierung 1}

  \begin{exampleblock}{Zur Erinnerung:}
    \begin{lstlisting}
< data SafeList a b where
<   Nil  :: SafeList a NotSafe
<   Cons :: a -> SafeList a b -> SafeList a (Safe b)
    \end{lstlisting}
  \end{exampleblock}

  \begin{exampleblock}{Unsichere Liste $\rightarrow$ Sichere Liste}
    \begin{lstlisting}
> makeSafe1 :: SafeList a b 
>              -> Maybe (SafeList a (Safe b))
> makeSafe1 Nil = Nothing
< makeSafe1 (Cons e l) = Just (Cons e l)  --- ???
    \end{lstlisting}
  \end{exampleblock}
\end{frame}




\begin{frame}[fragile]{Funktionen mit sicheren Listen: Konvertierung 2}

  \begin{exampleblock}{Zur Erinnerung:}
    \begin{lstlisting}
< data SafeList a b where
<   Nil  :: SafeList a NotSafe
<   Cons :: a -> SafeList a b -> SafeList a (Safe b)
    \end{lstlisting}
  \end{exampleblock}

  \begin{exampleblock}{Unsichere Liste $\rightarrow$ Sichere Liste}
    \begin{lstlisting}
> makeSafe2 :: SafeList a b 
>              -> a
>              -> SafeList a (Safe b)
> makeSafe2 l e = Cons e l
    \end{lstlisting}
  \end{exampleblock}

  \pause

  Möglich, aber identisch zu Cons

\end{frame}





\begin{frame}[fragile]{Funktionen mit sicheren Listen: Konvertierung 3}

  \begin{exampleblock}{Zur Erinnerung:}
    \begin{lstlisting}
< data SafeList a b where
<   Nil  :: SafeList a NotSafe
<   Cons :: a -> SafeList a b -> SafeList a (Safe b)
    \end{lstlisting}
  \end{exampleblock}

  \begin{exampleblock}{Sichere Liste $\rightarrow$ Unsichere Liste}
    \begin{lstlisting}
< makeUnsafe :: SafeList a (Safe b) 
<               -> SafeList a b
< makeUnsafe (Cons e l) = Cons e l -- ???
    \end{lstlisting}
  \end{exampleblock}


\pause

   Sinnvoll?

\end{frame}





















\subsection{Gemischte Listen}

\begin{frame}[fragile]{Gemischte Listen}

    Idee: Verschiedene Typen in einer Liste

    \pause
  
  \begin{sample}
    \begin{lstlisting}
< data MixList a b where
<   MNil  :: MixList a ()
<   MCons :: (Show c, Show b) =>
<         a -> MixList b c -> MixList a (MixList b c)
    \end{lstlisting}
  \end{sample}

  \begin{itemize}
  \item
    a ist der Typ des ersten Elements
  \item
    b ist der Typ der Restliste
  \end{itemize}

\end{frame}




\begin{frame}[fragile]{Gemischte Listen}

  \begin{exampleblock}{Zur Erinnerung:}
    \begin{lstlisting}
> data MixList a b where
>   MNil  :: MixList a ()
>   MCons :: (Show c, Show b) =>
>         a -> MixList b c -> MixList a (MixList b c)
    \end{lstlisting}
  \end{exampleblock}

  \begin{sample}
    \begin{lstlisting}
< > MCons 42 (MCons "hallo" MNil)
< 42 : "hallo" : []
< > :t MCons 42 (MCons "hallo" MNil)
< MixList t (MixList [Char] (MixList b ()))
    \end{lstlisting}
  \end{sample}

\end{frame}











\begin{comment} % nur für Compiler

> instance (Show a, Show b) => Show (MixList a b) where
>     show (MNil)      = "[]"
>     show (MCons e l) = show e ++ " : " ++ show l

\end{comment}









\subsection{Abstrakte Syntaxbäume}

\begin{frame}[fragile]{Unsichere Ausdrucksbäume (ADT)}

  \begin{exampleblock}{ADT:}
    \begin{lstlisting}
< data Expr = IntLiteral Int
<           | BoolLiteral Bool
<           | AndExpr Expr Expr
< inv :: Expr -> Bool
< inv (BoolLiteral _) = True
< inv (IntLiteral _)  = True
< inv (AndExpr l r)   = isInt l && isInt r
<                       `or` isBool l && isBool r
< -- isBool, isInt ...
    \end{lstlisting}
  \end{exampleblock}

  \begin{itemize}
  \item
    Umständliche Definition der Invariante (bei vielen Konstruktorfunktionen)
    \pause
  \item
    Kann mit ADT nur zur Laufzeit überprüft werden
  \item
    Ziel: Invariante durch Compiler überprüfen lassen
  \end{itemize}

\end{frame}






\begin{frame}[fragile]{Sichere Ausdrucksbäume (GADT)}

  \begin{exampleblock}{GADT:}
    \begin{lstlisting}
< data Expr a where
<   I     :: Int -> Expr Int
<   B     :: Bool -> Expr Bool
<   And   :: Expr Bool -> Expr Bool -> Expr Bool
    \end{lstlisting}
  \end{exampleblock}

\pause

  \begin{itemize}
  \item
    Invariante?
\pause
  \item
    Langweilig: Ausdrucksbäume bestehen nur aus Ints oder nur aus Bools
  \item
    Erweiterung: Vergleich und Addition
  \end{itemize}

\end{frame}









\begin{frame}[fragile]{Sichere Ausdrucksbäume (GADT)}

  \begin{exampleblock}{Erweiterung:}
    \begin{lstlisting}
> data Expr a where
>   I     :: Int -> Expr Int
>   B     :: Bool -> Expr Bool
>   And   :: Expr Bool -> Expr Bool -> Expr Bool
>   Equal :: (Show a, Eq a) =>
>            Expr a -> Expr a -> Expr Bool
>   Add   :: Expr Int -> Expr Int -> Expr Int
    \end{lstlisting}
  \end{exampleblock}

  \begin{itemize}
  \item
    Mischung von Ints und Bools \textbf{und} Invariante erfüllt
  \end{itemize}

\end{frame}





\begin{comment} % nur für Compiler

> instance Show a => Show (Expr a) where
>     show (I i) = show i
>     show (B b) = show b
>     show (And l r)   = "(" ++ show l ++ " and " ++ show r ++ ")"
>     show (Equal l r) = "(" ++ show l ++ " == " ++ show r ++ ")"
>     show (Add l r)   = "(" ++ show l ++ " + " ++ show r ++ ")"


\end{comment}








\begin{frame}[fragile]{Sichere Ausdrucksbäume: eval}

  \begin{exampleblock}{Zur Erinnerung:}
    \begin{lstlisting}
< data Expr a where
<   I     :: Int -> Expr Int
<   B     :: Bool -> Expr Bool
<   And   :: Expr Bool -> Expr Bool -> Expr Bool
<   Equal :: Eq a => Expr a -> Expr a -> Expr Bool
<   Add   :: Expr Int -> Expr Int -> Expr Int
    \end{lstlisting}
  \end{exampleblock}


  \begin{exampleblock}{Auswertung:}
    \begin{lstlisting}
> eval :: Expr a -> a
> eval (I i)       = i
> eval (B b)       = b
> eval (And l r)   = eval l && eval r
> eval (Equal l r) = eval l == eval r
> eval (Add l r)   = eval l + eval r
    \end{lstlisting}
  \end{exampleblock}



\end{frame}







\begin{frame}[fragile]{Sichere Ausdrucksbäume: optimize}

  \begin{exampleblock}{Zur Erinnerung:}
    \begin{lstlisting}
< data Expr a where
<   I     :: Int -> Expr Int
<   B     :: Bool -> Expr Bool
<   And   :: Expr Bool -> Expr Bool -> Expr Bool
<   Equal :: Eq a => Expr a -> Expr a -> Expr Bool
<   Add   :: Expr Int -> Expr Int -> Expr Int
    \end{lstlisting}
  \end{exampleblock}


  \begin{exampleblock}{optimize:}
    \begin{lstlisting}
> optimize :: Expr a -> Expr a
> optimize (And (B True) r) = optimize r
> optimize (And l r) = And (optimize l) (optimize r)
> optimize x = x
    \end{lstlisting}
  \end{exampleblock}



\end{frame}











\subsection{Vektoren}

\begin{frame}[fragile]{Vektoren}

  \begin{exampleblock}{Beispiel:}
    \begin{lstlisting}
> data Zero
> data Next a
> data Vec e n where
>   NullVec :: Vec e Zero
>   NextVec :: e -> Vec e n -> Vec e (Next n)
    \end{lstlisting}
  \end{exampleblock}

  \begin{exampleblock}{Konvertierung:}
    \begin{lstlisting}
> v2l :: Vec e n -> [e]
> v2l NullVec = []
> v2l (NextVec e l) = e : v2l l

< l2v :: [e] -> Vec e n -- ???
< l2v []    = NullVec
< l2v (e:l) = NextVec e (l2v l)
   \end{lstlisting}
  \end{exampleblock}

\end{frame}



\begin{comment}

> instance (Show a) => Show (Vec a n) where
>     show NullVec = "()"
>     show v       = "Vec [" ++ intercalate ", " (map show l') ++ "]"
>         where
>           l' = v2l v

\end{comment}




\begin{frame}[fragile]{Vektorfunktionen: Skalarprodukt}


  \begin{exampleblock}{Zur Erinnerung:}
    \begin{lstlisting}
< data Zero
< data Next a
< data Vec e n where
<   NullVec :: Vec e Zero
<   NextVec :: e -> Vec e n -> Vec e (Next n)
    \end{lstlisting}
  \end{exampleblock}

  \begin{exampleblock}{Skalarprodukt:}
    \begin{lstlisting}
> dot :: Vec Int n -> Vec Int n -> Int
> dot NullVec NullVec = 0
> dot (NextVec e1 l1) (NextVec e2 l2) = e1 * e2 + dot l1 l2
    \end{lstlisting}
  \end{exampleblock}

\end{frame}





\begin{frame}[fragile]{Vektorfunktionen: Konkatenation}

  \begin{exampleblock}{Zur Erinnerung:}
    \begin{lstlisting}
< data Zero
< data Next a
< data Vec e n where
<   NullVec :: Vec e Zero
<   NextVec :: e -> Vec e n -> Vec e (Next n)
    \end{lstlisting}
  \end{exampleblock}



  \begin{exampleblock}{Konkatenation:}
    \begin{lstlisting}
< concatV :: Vec e n1 -> Vec e n2 -> Vec e n3 -- ???
< concatV NullVec x         = x
< concatV (NextVec e1 l1) x = NextVec e1 (concatV l1 x)
    \end{lstlisting}
  \end{exampleblock}

\end{frame}
















\begin{frame}[fragile]{Vektoren -- zweiter Versuch:}

  \begin{exampleblock}{Beispiel:}
    \begin{lstlisting}
> data Wec e n where
>   Wec :: [e] -> Wec e n
    \end{lstlisting}
  \end{exampleblock}

  \begin{exampleblock}{Konvertierung:}
    \begin{lstlisting}
> w2l :: Wec e n -> [e]
> w2l (Wec l) = l

> l2w :: [e] -> Wec e n
> l2w l = Wec l
   \end{lstlisting}
  \end{exampleblock}

\end{frame}



\begin{frame}[fragile]{Vektoren -- zweiter Versuch: Invariante}

  \begin{exampleblock}{Zur Erinnerung:}
    \begin{lstlisting}
< data Wec e n where
<   Wec :: [e] -> Wec e n
    \end{lstlisting}
  \end{exampleblock}

  \begin{exampleblock}{Eigene Konstruktorfunktion:}
    \begin{lstlisting}
> w3 :: [e] -> Wec e (Next (Next (Next Zero)))
> w3 e = assert (length e == 3) (Wec e)
   \end{lstlisting}
  \end{exampleblock}

\end{frame}




\begin{comment}

> instance (Show a) => Show (Wec a n) where
>     show v       = "Wec [" ++ intercalate ", " (map show l') ++ "]"
>         where
>           l' = w2l v
\end{comment}










\begin{frame}[fragile]{Vektoren -- dritter Versuch:}

  \begin{exampleblock}{Beispiel:}
    \begin{lstlisting}
> data WecN e n where
>   Wec1 :: e -> WecN e (Next Zero)
>   Wec2 :: e -> e -> WecN e (Next (Next Zero))
>   Wec3 :: e -> e -> e -> WecN e (Next (Next (Next Zero)))
>   -- ...
    \end{lstlisting}
  \end{exampleblock}


\setbeamertemplate{itemize item}{$\approx$}

  \begin{itemize}
  \item
        Praktikabel?
  \end{itemize}

\setbeamertemplate{itemize item}{$\Rightarrow$$}

\end{frame}




















\section{Schlussbetrachtungen}



\subsection{Fazit}



\begin{frame}[fragile]{Fazit}

\setbeamertemplate{itemize item}{$+$}

  \begin{itemize}
  \item
    Mehr Typsicherheit

  \item
    Weniger Laufzeitfehler

  \item
    Geschicktere Invarianten

  \item
    Partielle Funktion $\Rightarrow$ totale Funktion

    \setbeamertemplate{itemize item}{$\approx$}
  
\pause

  \item
    $\ldots$ aber nicht alle

    \setbeamertemplate{itemize item}{$\approx$}
  
  \item
    Es gibt sinnvolle Funktionen, die nicht definiert werden können

\setbeamertemplate{itemize item}{$+$}
  \item
    Es gibt weniger sinnlose Funktionen, die definiert werden können
    
  \end{itemize}



\end{frame}








\subsection{Verwandtes: Associated Types}



\begin{frame}[fragile]{Associated Types (1)}

  \begin{itemize}
  \item
    Funktionen über Typen
  \item
    Erweiterung von Typklassen
  \end{itemize}

\pause

\begin{columns}

\column{.50\textwidth}

  \begin{exampleblock}{Herkömmliche Num-Typklasse:}
    \begin{lstlisting}
< class Num a where
<   (+++) :: a -> a -> a
< 
< 
< instance Num Int where
<   (+++) x y = x + y
< 
    \end{lstlisting}
  \end{exampleblock}

\vfill

\column{.50\textwidth}

  \begin{exampleblock}{Neue Num-Typklasse:}
    \begin{lstlisting}
> class MyNum a b where
>   type Sum a b :: *
>   (+++) :: a -> b -> Sum a b
> 
> instance MyNum Int Int where
>   type Sum Int Int = Int
>   (+++) x y = x + y
    \end{lstlisting}
  \end{exampleblock}

\vfill

\end{columns}

  \begin{itemize}
  \item
    Sum ist eine Funktion mit 2 Typen als Parameter und einem Typ als Rückgabewert
  \end{itemize}

\end{frame}






\begin{frame}[fragile]{Associated Types (2)}

  \begin{exampleblock}{Zur Erinnerung:}
    \begin{lstlisting}
< class MyNum a b where
<   type Sum a b :: *
<   (+++) :: a -> b -> Sum a b
    \end{lstlisting}
  \end{exampleblock}

  \begin{exampleblock}{MyNum Float Int:}
    \begin{lstlisting}
> instance Integral a => MyNum Float a where
>     type Sum Float a = Float
>     (+++) x y = x + intToFloat y
    \end{lstlisting}
  \end{exampleblock}

\pause

  \begin{itemize}
  \item
    Floats und Ints addieren!
\setbeamertemplate{itemize item}{$\approx$}
  \item
    Ints und Floats?
  \end{itemize}

\setbeamertemplate{itemize item}{$\Rightarrow$$}

\end{frame}






\begin{frame}[fragile]{Associated Types (3)}

  \begin{exampleblock}{Zur Erinnerung:}
    \begin{lstlisting}
< class MyNum a b where
<   type Sum a b :: *
<   (+++) :: a -> b -> Sum a b
    \end{lstlisting}
  \end{exampleblock}

  \begin{exampleblock}{MyNum Int Float:}
    \begin{lstlisting}
> instance Integral a => MyNum a Float where
>     type Sum a Float = Float
>     (+++) x y = y +++ x
    \end{lstlisting}
  \end{exampleblock}


  \begin{itemize}
  \item
    nun ist +++ kommutativ!
  \end{itemize}

\end{frame}





\subsection{Verwandtes: Dependent Types}



\begin{frame}[fragile]{Dependent Types (1)}

  \begin{itemize}
  \item
    Typen, die von Werten abhängen
  \item
    Agda
  \end{itemize}

\end{frame}





\begin{frame}[fragile]{Dependent Types (2)}

  \begin{exampleblock}{Willkürliches Beispiel:}
      q : $\Sigma N$  ($\lambda x \rightarrow$ if $x = 1$ then $N$ else $Bool$)
  \end{exampleblock}

  \begin{exampleblock}{Werte für q:}
      q = 4, true
      \\
      q = 1, 42
  \end{exampleblock}

\end{frame}




\begin{frame}[fragile]{Dependent Types (3)}
%\renewcommand{\labelitemi}{$\approx$}



  \begin{itemize}
  \item
    noch genauere Typüberpüfungen / Invarianten\\
    z.B.:
    \begin{itemize}
    \item
      Vektoren (auch mit GADTs)
    \item
      Matrizen ($m \times n \rightarrow n \times o \rightarrow m \times o$)
    \item
      Binäre Bäume (links $<$ mitte $<$ rechts)
    \end{itemize}


\setbeamertemplate{itemize item}{$\approx$}

\pause
  \item
    Vermischung Typ/Wert
  \item
    Endlosschleifen im Typsystem
  \item
    Typgleichheit
  \end{itemize}


\end{frame}


\setbeamertemplate{itemize item}{$\Rightarrow$}






\end{document}

